#!/bin/sh
#
# Script for generating coronerd traces.
#
# Copyright (C) 2014 Backtrace I/O.
# All Rights Reserved.
#
# All information contained herein is, and remains the property of Backtrace
# I/O and its suppliers, if any.  The intellectual and technical concepts
# contained herein are proprietary to Backtrace I/O and its suppliers and may
# be covered by U.S. and Foreign Patents, patents in process, and are protected
# by trade secret or copyright law.  Dissemination of this information or
# reproduction of this material is strictly forbidden unless prior written
# permission is obtained from Backtrace I/O Incorporated.
#

error()
{
	logger -p emerg "$@"
	exit $EXIT_FAILURE
}

usage()
{
	echo "Usage: coronerd-crash-report.sh <PID>" 1>&2
	exit $EXIT_FAILURE
}

PREFIX="/opt/backtrace"
PTRACE="${PREFIX}/bin/ptrace"
CORONER="${PREFIX}/bin/coroner"
SCRIPTS="${PREFIX}/coronerd/scripts/ptrace"
DIR="/var/backtrace/coronerd"

if test ! -x ${PTRACE}; then
	error "error: backtrace-ptrace is not installed."
fi

if test ! -x ${CORONER}; then
	error "error: backtrace-coroner is not installed."
fi

if test ! -d ${DIR}; then
	mkdir -p $DIR && chmod 700 $DIR
fi

if test $# -lt 1; then
	usage
fi

[ -r "/etc/default/coronerd" ] && . /etc/default/coronerd
[ -r "/etc/sysconfig/coronerd" ] && . /etc/sysconfig/coronerd

PID=$1
case $PID in
	''|*[!0-9]*)
		error "error: PID must be valid number."
		;;
	*)
		break
		;;
esac

PTRACE_ARGS="$PTRACE_ARGS --module-load=${SCRIPTS}/coronerd.lua"
PTRACE_ARGS="$PTRACE_ARGS --sandbox-whitelist=external_modules"
PTRACE_ARGS="$PTRACE_ARGS --load= --tls=current"
PTRACE_ARGS="$PTRACE_ARGS --kv=version:@cr_version_string:string_array"
PTRACE_ARGS="$PTRACE_ARGS --kv=libminidump_version:@libminidump_version_string:string_array"

has_asan_log=0
if [ -n "${CORONERD_ASAN_LOG_PATH}" ]; then
	asan_log="${CORONERD_ASAN_LOG_PATH}.${PID}"
	[ -f "$asan_log" ] && has_asan_log=1
	if [ "$has_asan_log" = "1" ]; then
		PTRACE_ARGS="$PTRACE_ARGS --attach=${asan_log}"
		sleep 3 # allow asan time to dump to the log
	fi
fi

$PTRACE $PTRACE_ARGS -O "${DIR}/coronerd" "${PID}"
RT=$?
kill -9 $PID
[ "$has_asan_log" = "1" ] && rm -f $asan_log
exit $RT
